import os
import gtk, gtk.glade

import nautilus
import dbus, dbus.glib

SUPPORTED_FORMAT = 'x-directory/normal'
APPLICATION_DBUS_IFACE='org.conduit.Application'
DATAPROVIDER_DBUS_IFACE="org.conduit.DataProvider"
CONDUIT_DBUS_IFACE="org.conduit.Conduit"
FLICKR_TWOWAY="FlickrTwoWay"
FOLDER_TWOWAY="FolderTwoWay"

FOLDER_TWOWAY_CONFIG ="<configuration><folder type='string'>%s</folder><folderGroupName type='string'>Home</folderGroupName><includeHidden type='bool'>False</includeHidden></configuration>"
FLICKR_TWOWAY_CONFIG ="<configuration><username type='string'>%s</username><photoSetName type='string'>%s</photoSetName><showPublic type='bool'>False</showPublic></configuration>"

class ConduitExtension(nautilus.MenuProvider):
    app = None
    folder_twoway = None
    flickr_twoway = None

    def __init__(self):
        bus = dbus.SessionBus()

        try:
            remote_object = bus.get_object(APPLICATION_DBUS_IFACE,"/")
            self.app = dbus.Interface(remote_object, APPLICATION_DBUS_IFACE)
        except dbus.exceptions.DBusException:
            print "Conduit unavailable"

    def menu_flickr_activate_cb(self, menu, folder):
        # it has got to be there
        if folder.is_gone ():
            return

        # get uri
        uri = folder.get_uri()

        # check if they needed providers are available
        dps = self.app.GetAllDataProviders()

        if not FOLDER_TWOWAY in dps and not FLICKR_TWOWAY in dps:
            return

        # setup flickr
        username, photoset_name = self._setup_flickr ()

        if not username:
            return

        # create dataproviders
        folder_twoway_path = self.app.GetDataProvider(FOLDER_TWOWAY)
        flickr_twoway_path = self.app.GetDataProvider(FLICKR_TWOWAY)

        bus = dbus.SessionBus()

        # set up folder config
        folder_twoway = bus.get_object(DATAPROVIDER_DBUS_IFACE, folder_twoway_path)
        folder_twoway.SetConfigurationXml(FOLDER_TWOWAY_CONFIG % uri)

        # set up flickr config
        flickr_twoway = bus.get_object(DATAPROVIDER_DBUS_IFACE, flickr_twoway_path)
        flickr_twoway.SetConfigurationXml(FLICKR_TWOWAY_CONFIG % (username, photoset_name))

        # now create conduit
        conduit_path = self.app.BuildConduit (folder_twoway_path, flickr_twoway_path)
        conduit = bus.get_object(CONDUIT_DBUS_IFACE, conduit_path)

        # do it to me, baby, real good!
        conduit.Sync()

    def get_file_items(self, window, files):
        # no conduit
        if not self.app:
            return

        # more than one selected?
        if len(files) != 1:
            return

        file = files[0]

        # must be a folder
        if not file.get_mime_type () == SUPPORTED_FORMAT:
            return

        # and we like local, although this might not be necessary
        if file.get_uri_scheme() != 'file':
            return

        # add the menu item
        item = nautilus.MenuItem('Conduit::synchronizeNautilus',
                                 'Synchronize to Flickr',
                                 'Synchronize folder with Flickr')

        item.connect('activate', self.menu_flickr_activate_cb, file)
        return item,


    def _setup_flickr(self):
        # this is not very good as it does not keep configuration
        # but you get the point
        dir = os.path.abspath(os.path.join(__file__, ".."))
        gladefile = os.path.join(dir, "config.glade")
        xml = gtk.glade.XML(os.path.join(dir, gladefile), "FlickrTwoWayConfigDialog")
        dlg = xml.get_widget("FlickrTwoWayConfigDialog")

        username_entry = xml.get_widget("username_entry")
        photoset_entry = xml.get_widget("photoset_entry")
        response = dlg.run ()

        username = None
        photoset_name = None

        if response == gtk.RESPONSE_OK:
            username = username_entry.get_text()
            photoset_name = photoset_entry.get_text()

        dlg.destroy ()
        return username, photoset_name
        
